/* Sample code to show how solving 4NF can be tricky. */
/*
	This example based on Louis Davidson, et al's Pro SQL
	Server 2005 Database Design and Optimization.

	We have as a setup a basic table with four attributes,
	of which the first three make up the primary key:
		- Room
		- Session
		- Presenter
		- PresentationTime

	We want to try to break this down into two
	separate tables and re-compose the data to match
	exactly what the single-table solution had.
*/

CREATE TABLE #RoomSessionPresenter
(
	Room VARCHAR(100),
	Session VARCHAR(100),
	Presenter VARCHAR(100),
	PresentationTime TIME
);

INSERT INTO #RoomSessionPresenter
(
	Room,
	Session,
	Presenter,
	PresentationTime
)
VALUES
('Alpha Room', 'Intro', 'Bob', '08:00:00'),
('Alpha Room', 'Intro', 'Chet', '08:00:00'),
('Beta Room', 'Intermediate', 'Janet', '09:00:00'),
('Gamma Room', 'Advanced', 'Bruce', '08:15:00'),
('Alpha Room', 'Recap', 'Bob', '11:00:00'),
('Alpha Room', 'Recap', 'Chet', '11:00:00'),
('Alpha Room', 'Recap', 'Janet', '11:00:00'),
('Alpha Room', 'Recap', 'Bruce', '11:00:00'),
('Alpha Room', 'After-Event', 'Bob', '14:00:00'),
('Alpha Room', 'After-Event', 'Janet', '14:00:00'),
('Beta Room', 'After-Event', 'Bob', '14:00:00'),
('Beta Room', 'After-Event', 'Janet', '14:00:00');

--Any solution we have must be able to build back up to this.
SELECT
	Room,
	Session,
	Presenter,
	PresentationTime
FROM #RoomSessionPresenter rsp;

/* First try:  Room-Session and Room-Presenter */
CREATE TABLE #RoomSession1
(
	Room VARCHAR(100),
	Session VARCHAR(100)
);

CREATE TABLE #RoomPresenter1
(
	Room VARCHAR(100),
	Presenter VARCHAR(100),
	PresentationTime TIME
);

INSERT INTO #RoomSession1
(
	Room,
	Session
)
SELECT DISTINCT
	Room,
	Session
FROM #RoomSessionPresenter;

INSERT INTO #RoomPresenter1
(
	Room,
	Presenter,
	PresentationTime
)
SELECT DISTINCT
	Room,
	Presenter,
	PresentationTime
FROM #RoomSessionPresenter;

--Let's build back up to the original query.
--We end up with 31 results, way more than we should have (11).
--This attempt is a failure.
SELECT
	rs.Room,
	rs.Session,
	rp.Presenter,
	rp.PresentationTime
FROM #RoomSession1 rs
	INNER JOIN #RoomPresenter1 rp
		ON rs.Room = rp.Room;

/* Second try:  Session-Room and Session-Presenter */
CREATE TABLE #SessionRoom2
(
	Session VARCHAR(100),
	Room VARCHAR(100)
);

CREATE TABLE #SessionPresenter2
(
	Session VARCHAR(100),
	Presenter VARCHAR(100),
	PresentationTime TIME
);

INSERT INTO #SessionRoom2
(
	Session,
	Room
)
SELECT DISTINCT
	Session,
	Room
FROM #RoomSessionPresenter;

INSERT INTO #SessionPresenter2
(
	Session,
	Presenter,
	PresentationTime
)
SELECT DISTINCT
	Session,
	Presenter,
	PresentationTime
FROM #RoomSessionPresenter;

--Success!  We get back the 12 rows we expected.
SELECT
	sr.Room,
	sr.Session,
	sp.Presenter,
	sp.PresentationTime
FROM #SessionRoom2 sr
	INNER JOIN #SessionPresenter2 sp
		ON sr.Session = sp.Session;


/* Third try:  Presenter-Room and Presenter-Session */
CREATE TABLE #PresenterRoom3
(
	Presenter VARCHAR(100),
	Room VARCHAR(100),
	PresentationTime TIME

);

CREATE TABLE #PresenterSession3
(
	Presenter VARCHAR(100),
	Session VARCHAR(100)
);

INSERT INTO #PresenterRoom3
(
	Presenter,
	Room,
	PresentationTime
)
SELECT DISTINCT
	Presenter,
	Room,
	PresentationTime
FROM #RoomSessionPresenter;

INSERT INTO #PresenterSession3
(
	Presenter,
	Session
)
SELECT DISTINCT
	Presenter,
	Session
FROM #RoomSessionPresenter;

--We end up with 32 rows, meaning we have phantom rows here.
SELECT
	pr.Room,
	ps.Session,
	pr.Presenter,
	pr.PresentationTime
FROM #PresenterRoom3 pr
	INNER JOIN #PresenterSession3 ps
		ON pr.Presenter = ps.Presenter;

/*
The only valid solution to this problem is solution #2,
in which we break up Session to be the "go-between" for
our binary entities.

So why is this the case?  Because of the data we have.
This is NOT a robust solution:  if you remove the After-Event
session for Janet in the Alpha Room (assuming that she'll
only be in the Beta Room), even this solution breaks down.

The reason that Session is our "pivot" entity is that
we're assured of the following things:
1)  A session may be held in multiple rooms at one time.
2)  A session may be held by multiple presenters at one time.
3)  If a session is held in multiple rooms and by multiple presenters
	at the same time, all relevant presenters will be in all relevant rooms.

If we relax pre-condition #3 (for example, by removing Janet from
the Alpha Room for the After-Event session), our resulting solution
ends up wanting to put her right back into the Alpha Room.

Two alternative sets of rules that could also work:
1a)  A session may be held in multiple rooms at one time.
2a)  Only one presenter may give a session at one time.

1b)  A session may be held in one room at one time.
2b)  A session may be held by multiple presenters at one time.

Doing either of these would ensure that we can "pivot" against Session.
*/