/* Sample code to show a 5NF failure. */
/*
	This example based on Louis Davidson, et al's Pro SQL
	Server 2005 Database Design and Optimization.

	We have as a setup a basic table with three attributes:
		- Teac her
		- Student
		- Class

	This setup is in 4NF because we cannot break the entity
	down into two lossless entities, so now we're trying
	to see if the entity is in 5NF by breaking it down
	into three entities.

	Our final output should be:
	Bob		Louis		Normalization
	Bob		Fred		T-SQL
	Larry	Fred		Normalization
*/
declare @TS table
(
	Teacher varchar(30),
	Student varchar(30)
);
declare @SC table
(
	Student varchar(30),
	Class varchar(30)
);
declare @TC table
(
	Teacher varchar(30),
	Class varchar(30)
);

insert into @TS(Teacher, Student) values
	('Bob', 'Louis'),
	('Bob', 'Fred'),
	('Larry', 'Fred');
insert into @SC(Student, Class) values
	('Louis', 'Normalization'),
	('Fred', 'T-SQL'),
	('Fred', 'Normalization');
insert into @TC(Teacher, Class) values
	('Bob', 'Normalization'),
	('Bob', 'T-SQL'),
	('Larry', 'Normalization');

select 
	ts.Teacher,
	ts.Student,
	tc.Class
from
	@TS ts
	inner join @SC sc on ts.Student = sc.Student
	inner join @TC tc 
		on ts.Teacher = tc.Teacher
		and sc.Class = tc.Class;
	
/* Sample code to show a 5NF success. */
/*
	This example based on William Kent's A Simple Guide to Five
	Normal Forms in Relational Database Theory.
	http://www.bkent.net/Doc/simple5.htm#label4.2

	We have as a setup a basic table with three attributes:
		- Agent
		- Company
		- Product

	This setup is in 4NF because we cannot break the entity
	down into two lossless entities, so now we're trying
	to see if the entity is in 5NF by breaking it down
	into three entities.

	We are working with an important assumption:  a symmetric
	constraint is in place.  This is a BUSINESS rule which
	tells us, in our scenario, that if Smith sells Fords,
	if Smith sells Cars, and if Ford sells Cars, Smith must
	therefore sell Cars for Ford.

	The above example did NOT have a symmetric constraint in place,
	so a breakdown into three separate tables was lossy.

	Our final output should be:
	Agent	Company	Product
	-----	-------	------
	Smith	Ford	Car
	Smith	Ford	Truck
	Smith	GM		Car
	Smith	GM		Truck
	Jones	Ford	Car
	Jones	Ford	Truck
	Brown	Ford	Car
	Brown	GM		Car
	Brown	Toyota	Bus
	Brown	Toyota	Car
*/
declare @AC table
(
	Agent varchar(30),
	Company varchar(30)
);
declare @CP table
(
	Company varchar(30),
	Product varchar(30)
);
declare @AP table
(
	Agent varchar(30),
	Product varchar(30)
);

insert into @AC(Agent, Company) values
	('Smith', 'Ford'),
	('Smith', 'GM'),
	('Jones', 'Ford'),
	('Brown', 'Ford'),
	('Brown', 'GM'),
	('Brown', 'Toyota');
insert into @CP(Company, Product) values
	('Ford', 'Car'),
	('Ford', 'Truck'),
	('GM', 'Car'),
	('GM', 'Truck'),
	('Toyota', 'Car'),
	('Toyota', 'Bus');
insert into @AP(Agent, Product) values
	('Smith', 'Car'),
	('Smith', 'Truck'),
	('Jones', 'Car'),
	('Jones', 'Truck'),
	('Brown', 'Car'),
	('Brown', 'Bus');
	
select
	ap.Agent,
	ac.Company,
	ap.Product
from
	@AC ac
	inner join @CP cp on ac.Company = cp.Company
	inner join @AP ap 
		on ac.Agent = ap.Agent
		and cp.Product = ap.Product
order by
	ap.Agent desc,
	ac.Company asc,
	ap.Product asc;